package WxTohoho::App;

use base qw(Wx::App);

sub OnInit {
    my ( $self ) = @_;

    my $res = WxTohoho::XRC->instance;
    my $top_frame = $res->LoadFrame(undef, "TopFrame");
    $top_frame->Show(1);

    return 1;
}

package WxTohoho::XmlResouceHandler;

use strict;
use warnings;
use Wx::XRC;
use base 'Wx::PlXmlResourceHandler';

our $LAST_NODE;
our $LAST_OBJECT_NODE;

sub CanHandle {
    my( $self, $xmlnode ) = @_;
    $LAST_OBJECT_NODE = $LAST_NODE = $xmlnode;
    return 0;
}

sub last_object_node {
    _descend($LAST_NODE);
    $LAST_OBJECT_NODE;
}

sub _descend {
    my $node = shift;

    my $tail = do {
        my $tmp = $node;
        while ( $tmp->GetNext ) {
            $tmp = $tmp->GetNext;
            if ( $tmp->GetName eq "object" ) {
                $LAST_OBJECT_NODE = $tmp;
            }
        }
        $tmp;
    };

    my $child = $tail->GetChildren;
    $child ? _descend($child) : $tail;
}

package WxTohoho::SubclassFactory;

use Wx::Event qw(EVT_WINDOW_CREATE);
use base qw(Wx::XmlSubclassFactory);

sub Create {
    my ( $self, $subclass ) = @_;

    my $window = $subclass->new;

    my $last = WxTohoho::XmlResouceHandler->last_object_node();
    my $last_window_name = $last->GetPropVal("name", "");
    my $last_window_id   = Wx::XmlResource::GetXMLID($last_window_name);

    if ( $window->can("OnCreate") ) {
        my $code = sub {
            my ( $this, $evt ) = @_;

            if ( $evt->GetEventObject->GetId == $last_window_id ) {
                EVT_WINDOW_CREATE( $this, -1, undef );
                shift;
                $this->OnCreate(@_);
            }

            $evt->Skip;
        };
        EVT_WINDOW_CREATE( $window, -1, $code );
    }

    return $window;
}

package WxTohoho::XRC;

use File::Spec::Functions;

Wx::XmlResource::AddSubclassFactory( WxTohoho::SubclassFactory->new );

our $INSTANCE;

sub _instance {
    my $res = Wx::XmlResource->new;

    my $xrc_file = $ENV{PAR_TEMP} ? catfile($ENV{PAR_TEMP}, "inc", "WxTohoho.xrc") : "WxTohoho.xrc";

    $res->AddHandler( WxTohoho::XmlResouceHandler->new );
    $res->InitAllHandlers();
    $res->Load($xrc_file) or die "xrc file not found. $xrc_file";

    $res;
}

sub instance {
    $INSTANCE ||= _instance();    
}

package WxTohoho::ListCtrl;

use base qw(Wx::ListCtrl);
use Wx qw(:listctrl);

sub OnCreate {
    my ( $self ) = @_;

    $self->InsertColumn($_-1, "Column $_", wxLIST_FORMAT_LEFT, -1) for ( 1 .. 5);
    $self->SetItemCount( 100 );
}

sub OnGetItemText {
    my ( $self, $row, $col ) = @_;
    sprintf("row %02i, col %i", $row, $col);
}

package WxTohoho::StaticText;

use strict;
use base 'Wx::StaticText';
use Wx qw(:colour);

sub new {
    my ( $class, $parent, $id, $label ) = @_;
    $class->SUPER::new( $parent, $id || -1, $label || "" );
}

sub OnCreate {
    my (
    shift->SetForegroundColour( wxRED );
}

package main;

my $app = WxTohoho::App->new;
$app->MainLoop;
